
    ########   m#####m  ##m###m   ##m###m    m#####m        @@@@@@@@  
        m#"    " mmm##  ##"  "##  ##"  "##   " mmm##      @@@@@@@@@@@@
      m#"     m##"""##  ##    ##  ##    ##  m##"""##      @@@      @@@
    m##mmmmm  ##mmm###  ###mm##"  ###mm##"  ##mmm###      @@  @@@@  @@
    """"""""   """" ""  ## """    ## """     """" ""          @@@@    
                        ##        ##                 

### Razor-sharp DSL for modern web apps

Zappa is a CoffeeScript DSLish layer on top of express, socket.io and other libs to build web apps free of Boilerplate Stuff.

It is heavily influenced by [that legendary framework](http://www.sinatrarb.com) named after another awesome Frank, and also a bit by [Camping](), but tries to go beyond simply mimicking them and explore interesting possibilities opened by Node.js: trivialized websockets, client-server integration, multi-app files, etc.

### Hi, World

In app.coffee:

    get '/': 'hi'

Then:

    $ npm install zappa
    $ zappa app.coffee

Open [localhost:8000](http://localhost:8000). Profit.

If you want to package you app CoffeeScript-free and executable by the standard `node` command, just compile it and add a zappup.js:

    require('zappa').run('app.js')

And:

    $ coffee -c app.coffee
    $ node zappup.js

#### OK, but static one-line strings as response are mostly useless. Can you show me something resembling more of a real web app?

    get '*': '''
      <!DOCTYPE html>
      <html>
        <head><title>Sorry, we'll be back soon!</title></head>
        <body><h1>Sorry, we'll be back soon!</h1></body>
      </html>
    '''

#### Seriously.

    get '/:name': ->
      "Hi, #{@name}"

#### Let me guess. You can also post/put/del, use regexes, routes are matched first to last, all like any sinatra clone worth its salt.

Exactly. Actually, zappa hands down all the serious work to express, then gets the credit. So generally if it works in express, it works in zappa:

    get '/': 'got'
    post '/': 'posted'
    put '/': 'put'
    del '/': 'deleted'
    get /bacon$/, -> 'yum'
    get '/pass': -> next()

#### Fine. But this is node. What about some async?

    get '/:name': ->
      actbusy =>
        send "...oh you're there, hi #{@name}"

    # or if you don't want to use the fat arrow to change '@/this':

    get '/:name': ->
      actbusy ->
        send "...oh you're there, hi #{data.name}"

####

    del: '/widgets/:id': ->
      redirect '/login' unless is_admin()
      Widgets.destroy @id, ->
        redirect '/widgets'

Three in a row:

    get
      '/widgets': ->
        Widgets.all (@widgets) =>
          render 'widgets/list'

      '/widgets/new': ->
        render 'widgets/new'

      '/widgets/edit/:id': ->
        Widgets.get @id, (@widget) =>
          render 'widgets/edit'

    get
      '/foo': 'bar'
      '/ping': 'pong'

Views:

    get '/': ->
      @title = 'Views example'
      render 'index'

    layout ->
      doctype 5
      html ->
        head ->
          title @title
      body ->
        @content

    template index: ->
      h1 @title

Named layout:

    get '/admin': ->
      @title 'Admin'
      render 'index', layout: 'admin'
    
    layout admin: ->
      doctype 5
      html ->
        head ->
          title @title + ' | Admin'
      body ->
        @content

    template 'widgets/new' ->
      form action: '/', method: 'post', ->
        input name: 'name'
        button 'New'

Post rendering with server-side jQuery:

    postrender roles: ->
      $('.admin').remove() if @role isnt 'admin'
      $('.' + @role).css 'color', '#f00'

    get '/': ->
      @role = 'janitor'
      render 'index', apply: 'roles'

Scope:

    @abc = '123'
    abc = '123'

    get '/': ->
      @abc # undefined
      abc # undefined

Def:

    def abc: '123'
    def func: ->
      request # undefined
      'func'

    get '/': ->
      abc # 123
      func() # 'func'

Helpers:

    helper role: (name) ->
      redirect '/login' unless @user.role is name

    get '/admin': ->
      role 'admin'
      # do stuff

Using:

    using 'eco' # same as def eco: require 'eco'

    get '/': ->
      eco.version

Settings:

    set foo: 'bar'

    set foo: 'bar', ping: 'pong'

    set
      foo: 'bar'
      ping: 'pong'

    get '/': ->
      settings.foo

WebSockets / Comet:

    get '/chat': ->
      render 'chat'

    at connection ->
      send 'hi'
      broadcast "#{client.id} connected"

    msg said: ->
      broadcast 'said', id: @id, text: @text

Client-server integration:

    get '/jquery.js': -> js 'jquery'
    client '/jquery.js': 'jquery'
    client '/index.js': 'socket.io', def: 'client/index', render: 'roles'

    get '/': ->
      template ->
        doctype 5
        html ->
          head ->
            title 'bla'
          script src: '/jquery.js'
          script src: '/index.js'
          

Multiple apps:

    app 'blog'
    get '/': 'blog'

    app 'chat'
    get '/': 'chat'

    $ zappa apps.coffee
    => App "blog" started on port 8000
    => App "chat" started on port 8001

    or:

    $ zappa -p 4567,4568,4569 apps.coffee

Too crowded?

    include 'model.coffee'
    include 'controllers/http.coffee'
    include 'controllers/websockets.coffee'
    include 'controllers/client.coffee'
    include 'controllers/common.coffee'
    include 'views.coffee'

Or by subject:

    include 'users.coffee'
    include 'widgets.coffee'
    include 'gadgets.coffee'
    include 'funzos.coffee'
